<?php

namespace Seb\Baltic\Helper;

use Seb\Baltic\Model\Config;
use Magento\Framework\App\ResourceConnection;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;

class OrderPaymentMethod
{
    /**
     * @var Config
     */
    protected $config;

    /**
     * @var OrderInterface
     */
    protected $orderInterface;

    /**
     * @var OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var ResourceConnection
     */
    protected $resourceConnection;

    /**
     * OrderPaymentMethod constructor.
     *
     * @param Config $config
     * @param OrderInterface $orderInterface
     * @param OrderRepositoryInterface $orderRepository
     * @param ResourceConnection $resourceConnection
     */
    public function __construct(
        Config $config,
        OrderInterface $orderInterface,
        OrderRepositoryInterface $orderRepository,
        ResourceConnection $resourceConnection
    ) {
        $this->config = $config;
        $this->orderInterface = $orderInterface;
        $this->orderRepository = $orderRepository;
        $this->resourceConnection = $resourceConnection;
    }

    /**
     * Update the payment method title based on the order.
     *
     * @param $order
     * @return void
     */
    public function updateByOrder($order)
    {
        $additionalInfo = $order->getPayment()->getAdditionalInformation();
        if (empty($additionalInfo)) {
            return;
        }

        if (empty($additionalInfo['selectedMethod'])) {
            $additionalInfo['selectedMethod'] = 'card';
        }

        $additionalInfo['title'] = $this->getPaymentMethodTitle($additionalInfo['selectedMethod']);

        $this->updateOrder($order, $additionalInfo);
        $this->updateOrderGrid($order, $additionalInfo);
    }

    /**
     * Update the payment method title based on the order reference.
     *
     * @param string $orderReference
     * @param string $selectedMethod
     * @return void
     */
    public function updateByOrderReference($orderReference, $selectedMethod = 'card')
    {
        $order = $this->orderInterface->loadByIncrementId($orderReference);
        $additionalInfo = [
            'title' => $this->getPaymentMethodTitle($selectedMethod),
        ];

        $this->updateOrder($order, $additionalInfo);
        $this->updateOrderGrid($order, $additionalInfo);
    }

    /**
     * Update the order table.
     *
     * @param $order
     * @param array $additionalInfo
     *
     * @return void
     */
    private function updateOrder($order, $additionalInfo)
    {
        try {
            $order->setData('baltic_payment_method', $additionalInfo['title']);
            $this->orderRepository->save($order);
        } catch (\Exception $e) {}
    }

    /**
     * Update the order grid table.
     *
     * @param $order
     * @param array $additionalInfo
     *
     * @return void
     */
    private function updateOrderGrid($order, $additionalInfo)
    {
        $connection = $this->resourceConnection->getConnection();
        $tableName = $this->resourceConnection->getTableName('sales_order_grid');

        $connection->update(
            $tableName,
            ['baltic_payment_method' => $additionalInfo['title']],
            ['entity_id = ?' => (int)$order->getId()]
        );
    }

    /**
     * Get the payment method title based on the stored methods.
     *
     * @param $selectedMethod
     *
     * @return string
     */
    private function getPaymentMethodTitle($selectedMethod = 'card')
    {
        if ($selectedMethod === 'google_pay' || $selectedMethod === 'apple_pay' || $selectedMethod === 'click_to_pay') {
            $selectedMethod = 'card';
        }

        $paymentMethods = $this->config->getPaymentMethods();
        $defaultTitle = $this->config->getTitle();

        foreach ($paymentMethods as $paymentMethod) {
            foreach ($paymentMethod as $method) {
                if ($method['source'] === $selectedMethod) {
                    return $method['display_name'];
                }
            }
        }

        return $defaultTitle;
    }
}
