define([
    'ko',
    'uiComponent',
    'Seb_Baltic/js/model/method',
    'Magento_Checkout/js/model/quote',
    'Magento_Checkout/js/model/full-screen-loader',
    'mage/url',
    'mage/storage'
], function(ko, Component, method, quote, fullScreenLoader, urlBuilder, storage) {
    "use strict";

    var methodReloadTimer;

    return Component.extend({
        selectedMethod: ko.observable(null),
        selectedCountry: ko.observable(window.checkoutConfig.payment.baltic.defaultCountry),
        methodCollection: [],
        cards: ko.observable(null),
        banks: ko.observable(null),
        alternatives: ko.observable(null),
        applePayAvailable: window.checkoutConfig.payment.baltic.applepay_available,
        googlePayAvailable: window.checkoutConfig.payment.baltic.googlepay_available,
        amount: ko.observable(window.checkoutConfig.payment.baltic.amount),

        initialize: function() {
            this._super();
            this.setMethods();
            this.selectCountry(window.checkoutConfig.payment.baltic.defaultCountry);
            this.initSubscribers();
        },
        setMethods: function() {
            var methods = method.prototype.fromData(window.checkoutConfig.payment.baltic.methods);
            this.setCollection(methods);
            this.checkSelectedMethod();
            this.cards(this.getCards());
            this.banks(this.getBanks());
            this.alternatives(this.getAlternatives());
            this.selectCountry(this.selectedCountry());
        },
        checkSelectedMethod: function () {
            var self = this;
            if(self.getSelectedMethod()) {
                var newSelectedMethod = null;
                self.getCollection().forEach(function(method) {
                    if(method.getId() == self.selectedMethod().getId()) {
                        newSelectedMethod = method;
                    }
                });
                self.selectedMethod(newSelectedMethod);
            }
        },
        initSubscribers: function () {
            var self = this;
            quote.totals.subscribe(
                function () {
                    if(quote.totals().base_grand_total != self.amount()) {
                        self.prepareReload();
                    }
                }
            );
        },
        prepareReload: function() {
            var self = this;
            clearTimeout(methodReloadTimer);
            methodReloadTimer = setTimeout(function() { self.reloadMethods(); }, 200);
        },
        reloadMethods: function() {
            var self = this;
            var url = urlBuilder.build("baltic/methods/index");

            fullScreenLoader.startLoader();
            storage.get(
                url
            ).done(
                function (response) {
                    window.checkoutConfig.payment.baltic.methods = response.methods;
                    self.setMethods();
                    window.checkoutConfig.payment.baltic.countries = response.countries;
                    window.dispatchEvent(new Event('balticCountries'));
                    self.amount(response.amount);
                    fullScreenLoader.stopLoader();
                }
            ).fail(
                function (response) {
                    fullScreenLoader.stopLoader();
                }
            );
        },
        setCollection: function(collection) {
            this.methodCollection = collection;
        },
        getCollection: function() {
            return this.methodCollection;
        },
        getMethods: function(type) {
            switch (type) {
                case this.getCardsCode():
                    return this.cards();
                    break;
                case this.getBanksCode():
                    return this.banks();
                    break;
                case this.getAlternativeCode():
                    return this.alternatives();
                    break;
                default:
                    return [];
            }
        },
        getCards: function() {
            var self = this;
            return this.getCollection().filter(function(method) {
                return method.getType() === self.getCardsCode();
            });
        },
        getBanks: function() {
            var self = this;
            return this.getCollection().filter(function(method) {
                return method.getType() === self.getBanksCode();
            });
        },
        getAlternatives: function() {
            var self = this;
            return this.getCollection().filter(function(method) {
                return method.getType() === self.getAlternativeCode();
            });
        },
        selectMethod: function(method) {
            var self = this;
            return function() {
                self.selectedMethod(method);
                return true;
            }
        },
        getSelectedMethod: function() {
            return this.selectedMethod();
        },
        isCardSelected: function() {
            return this.selectedMethod() ? this.selectedMethod().getType() === 'cards' : false;
        },
        isAppleSelected: function() {
            return this.selectedMethod() ? this.selectedMethod().getType() === 'apple' : false;
        },
        isGoogleSelected: function() {
            return this.selectedMethod() ? this.selectedMethod().getType() === 'google' : false;
        },
        isMethodSelected: function(method) {
            return this.selectedMethod() ? this.selectedMethod().getId() === method.getId() : false;
        },
        isCardsVisible: function() {
            return this.isTypeVisible(this.getCardsCode());
        },
        isBanksVisible: function() {
            return this.isTypeVisible(this.getBanksCode());
        },
        isAlternativesVisible: function() {
            return this.isTypeVisible(this.getAlternativeCode());
        },
        isAppleVisible: function() {
            if(window.ApplePaySession) {
                if(ApplePaySession.canMakePayments()) {
                    return this.applePayAvailable;
                }
            }
            return false;
        },
        isGoogleVisible: function() {
            return this.googlePayAvailable;
        },
        isTypeVisible: function(type) {
            return this.getVisibleMethods(type).length > 0;
        },
        getVisibleMethods: function(type) {
            var collection = this.getMethods(type);
            if (collection.length > 0) {
                return collection.filter(function(method) {
                    return method.isVisible();
                })
            }
            return [];
        },
        getVisibleMethodsCount: function() {
            return this.isCardsVisible() + this.isBanksVisible() + this.isAlternativesVisible() +
                this.isAppleVisible() + this.isGoogleVisible();
        },
        selectCountry: function(country) {
            this.getCollection().forEach(function(method) {
                method.setVisibility(
                    method.getCountry() == null ||
                    method.getCountry() == 'N/A' ||
                    method.getCountry() === country
                );
            });
        },
        getCardsCode: function() {
            return 'cards';
        },
        getBanksCode: function() {
            return 'banks';
        },
        getAlternativeCode: function() {
            return 'alternative';
        },
        getAppleCode: function() {
            return 'apple';
        },
        getGoogleCode: function() {
            return 'google';
        },
        getAppleButtonStyle: function() {
            return window.checkoutConfig.payment.baltic.appleButton
        },
        getGoogleButtonStyle: function() {
            return window.checkoutConfig.payment.baltic.googleButton
        }
    });
});
